@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -65,7 +65,7 @@ static NV_STATUS test_ordering(uvm_va_space_t *va_space)
     if (!at_least_one_gk110_plus)
         return NV_OK;
 
-    gpu = uvm_processor_mask_find_first_gpu(&va_space->registered_gpus);
+    gpu = uvm_va_space_find_first_gpu(va_space);
     TEST_CHECK_RET(gpu != NULL);
 
     status = uvm_rm_mem_alloc_and_map_all(gpu, UVM_RM_MEM_TYPE_SYS, buffer_size, &mem);
@@ -310,15 +310,19 @@ static uvm_channel_type_t random_channel_type(uvm_test_rng_t *rng)
     return (uvm_channel_type_t)uvm_test_rng_range_32(rng, 0, UVM_CHANNEL_TYPE_COUNT - 1);
 }
 
-static uvm_gpu_t *random_gpu(uvm_test_rng_t *rng, const uvm_processor_mask_t *mask)
+static uvm_gpu_t *random_va_space_gpu(uvm_test_rng_t *rng, uvm_va_space_t *va_space)
 {
     uvm_gpu_t *gpu;
-    NvU32 gpu_count = uvm_processor_mask_get_gpu_count(mask);
+    NvU32 gpu_count = uvm_processor_mask_get_gpu_count(&va_space->registered_gpus);
     NvU32 gpu_index = uvm_test_rng_range_32(rng, 0, gpu_count - 1);
+
     UVM_ASSERT(gpu_count > 0);
-    for_each_gpu_in_mask(gpu, mask)
+
+    for_each_va_space_gpu(gpu, va_space) {
         if (gpu_index-- == 0)
             return gpu;
+    }
+
     UVM_ASSERT(0);
     return NULL;
 }
@@ -356,12 +360,11 @@ static NV_STATUS stress_test_all_gpus_in_va(uvm_va_space_t *va_space,
     uvm_gpu_t *gpu;
     NvU32 i, j;
     uvm_test_stream_t *streams;
-    const uvm_processor_mask_t *gpu_mask = &va_space->registered_gpus;
     uvm_test_rng_t rng;
 
     uvm_test_rng_init(&rng, seed);
 
-    gpu = uvm_processor_mask_find_first_gpu(&va_space->registered_gpus);
+    gpu = uvm_va_space_find_first_gpu(va_space);
     TEST_CHECK_RET(gpu != NULL);
 
     streams = uvm_kvmalloc_zero(sizeof(*streams) * num_streams);
@@ -441,7 +444,7 @@ static NV_STATUS stress_test_all_gpus_in_va(uvm_va_space_t *va_space,
         for (j = 0; j < num_streams; ++j) {
             uvm_test_stream_t *stream = &streams[j];
             uvm_channel_type_t channel_type = random_channel_type(&rng);
-            gpu = random_gpu(&rng, gpu_mask);
+            gpu = random_va_space_gpu(&rng, va_space);
 
             if (fatal_signal_pending(current)) {
                 status = NV_ERR_SIGNAL_PENDING;
@@ -455,7 +458,7 @@ static NV_STATUS stress_test_all_gpus_in_va(uvm_va_space_t *va_space,
                                             "stream %u payload %u gid %u channel_type %u",
                                             j,
                                             stream->queued_counter_value,
-                                            gpu->id,
+                                            uvm_id_value(gpu->id),
                                             channel_type);
             TEST_CHECK_GOTO(status == NV_OK, done);
 
@@ -636,7 +639,7 @@ static NV_STATUS uvm_test_channel_stress_update_channels(uvm_va_space_t *va_spac
     uvm_va_space_down_read(va_space);
 
     for (i = 0; i < params->iterations; ++i) {
-        uvm_gpu_t *gpu = random_gpu(&rng, &va_space->registered_gpus);
+        uvm_gpu_t *gpu = random_va_space_gpu(&rng, va_space);
         uvm_channel_manager_update_progress(gpu->channel_manager);
 
         if (fatal_signal_pending(current)) {
@@ -666,7 +669,7 @@ static NV_STATUS uvm_test_channel_noop_push(uvm_va_space_t *va_space,
 
     for (i = 0; i < params->iterations; ++i) {
         uvm_channel_type_t channel_type = random_channel_type(&rng);
-        gpu = random_gpu(&rng, &va_space->registered_gpus);
+        gpu = random_va_space_gpu(&rng, va_space);
 
         status = uvm_push_begin(gpu->channel_manager, channel_type, &push, "noop push");
         if (status != NV_OK)
@@ -686,7 +689,7 @@ static NV_STATUS uvm_test_channel_noop_push(uvm_va_space_t *va_space,
     if (params->verbose > 0)
         UVM_TEST_PRINT("Noop pushes: completed %u pushes seed: %u\n", i, params->seed);
 
-    for_each_gpu_in_mask(gpu, &va_space->registered_gpu_va_spaces) {
+    for_each_va_space_gpu_in_mask(gpu, va_space, &va_space->registered_gpu_va_spaces) {
         NV_STATUS wait_status = uvm_channel_manager_wait(gpu->channel_manager);
         if (status == NV_OK)
             status = wait_status;
