@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -459,10 +459,10 @@ bool __uvm_check_unlocked_order(uvm_lock_order_t lock_order);
 // when checking for a trylock.
 bool __uvm_check_lockable_order(uvm_lock_order_t lock_order, uvm_lock_flags_t flags);
 
-// Check that all locks have been released in a uvm thread context
-bool __uvm_check_all_unlocked(uvm_thread_context_t *uvm_context);
+// Check that all locks have been released in a thread context lock
+bool __uvm_check_all_unlocked(uvm_thread_context_lock_t *context_lock);
 
-// Check that all locks have been released in the current uvm thread context
+// Check that all locks have been released in the current thread context lock
 bool __uvm_thread_check_all_unlocked(void);
 
 // Check that the locking infrastructure has been initialized
@@ -636,9 +636,17 @@ typedef struct
 #endif
 } uvm_rw_semaphore_t;
 
-#define uvm_assert_rwsem_locked_mode(uvm_sem, flags) ({                             \
-        typeof(uvm_sem) _sem = (uvm_sem);                                           \
-        UVM_ASSERT(rwsem_is_locked(&_sem->sem) && uvm_check_locked(_sem, (flags))); \
+//
+// Note that this is a macro, not an inline or static function so the
+// "uvm_sem" argument is subsituted as text. If this is invoked with
+// uvm_assert_rwsem_locked_mode(_sem, flags) then we get code "_sem = _sem"
+// and _sem is initialized to NULL. Avoid this by using a name unlikely to
+// be the same as the string passed to "uvm_sem".
+// See uvm_down_read() and uvm_up_read() below as examples.
+//
+#define uvm_assert_rwsem_locked_mode(uvm_sem, flags) ({                               \
+        typeof(uvm_sem) _sem_ = (uvm_sem);                                            \
+        UVM_ASSERT(rwsem_is_locked(&_sem_->sem) && uvm_check_locked(_sem_, (flags))); \
     })
 
 #define uvm_assert_rwsem_locked(uvm_sem) \
@@ -660,38 +668,35 @@ static void uvm_init_rwsem(uvm_rw_semaphore_t *uvm_sem, uvm_lock_order_t lock_or
     uvm_assert_rwsem_unlocked(uvm_sem);
 }
 
-static void __uvm_down_read(uvm_rw_semaphore_t *uvm_sem)
-{
-    down_read(&uvm_sem->sem);
-    uvm_assert_rwsem_locked_read(uvm_sem);
-}
-
-#define uvm_down_read(sem) ({                              \
-        typeof(sem) _sem = (sem);                          \
+#define uvm_down_read(uvm_sem) ({                          \
+        typeof(uvm_sem) _sem = (uvm_sem);                  \
         uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
-        __uvm_down_read(_sem);                             \
+        down_read(&_sem->sem);                             \
+        uvm_assert_rwsem_locked_read(_sem);                \
     })
 
-static void __uvm_up_read(uvm_rw_semaphore_t *uvm_sem)
-{
-    uvm_assert_rwsem_locked_read(uvm_sem);
-    up_read(&uvm_sem->sem);
-}
-#define uvm_up_read(sem) ({                                  \
-        typeof(sem) _sem = (sem);                            \
-        __uvm_up_read(_sem);                                 \
+#define uvm_up_read(uvm_sem) ({                              \
+        typeof(uvm_sem) _sem = (uvm_sem);                    \
+        uvm_assert_rwsem_locked_read(_sem);                  \
+        up_read(&_sem->sem);                                 \
         uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
     })
 
-static void __uvm_down_write(uvm_rw_semaphore_t *uvm_sem)
-{
-    down_write(&uvm_sem->sem);
-    uvm_assert_rwsem_locked_write(uvm_sem);
-}
-#define uvm_down_write(sem) ({                                \
-        typeof (sem) _sem = (sem);                            \
+// Unlock w/o any tracking. This should be extremely rare and *_no_tracking
+// helpers will be added only as needed.
+//
+// TODO: Bug 2594854:
+// TODO: Bug 2583279: Remove macro when bugs are fixed
+#define uvm_up_read_no_tracking(uvm_sem) ({                  \
+        typeof(uvm_sem) _sem = (uvm_sem);                    \
+        up_read(&_sem->sem);                                 \
+    })
+
+#define uvm_down_write(uvm_sem) ({                            \
+        typeof (uvm_sem) _sem = (uvm_sem);                    \
         uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
-        __uvm_down_write(_sem);                               \
+        down_write(&_sem->sem);                               \
+        uvm_assert_rwsem_locked_write(_sem);                  \
     })
 
 // trylock for reading: returns 1 if successful, 0 if not.  Out-of-order lock
@@ -699,70 +704,56 @@ static void __uvm_down_write(uvm_rw_semaphore_t *uvm_sem)
 // allow it.  However, if an out-of-order lock acquisition attempt fails, it is
 // the caller's responsibility to back off at least to the point where the
 // next held lower-order lock is released.
-static int __uvm_down_read_trylock(uvm_rw_semaphore_t *uvm_sem)
-{
-    int ret = down_read_trylock(&uvm_sem->sem);
-    if (ret == 0)
-        return ret;
-
-    uvm_assert_rwsem_locked_read(uvm_sem);
-
-    return ret;
-}
-#define uvm_down_read_trylock(sem) ({                                               \
-        typeof(sem) _sem = (sem);                                                   \
+#define uvm_down_read_trylock(uvm_sem) ({                                           \
+        typeof(uvm_sem) _sem = (uvm_sem);                                           \
         int locked;                                                                 \
         uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_SHARED | UVM_LOCK_FLAGS_TRYLOCK); \
-        locked = __uvm_down_read_trylock(_sem);                                     \
+        locked = down_read_trylock(&_sem->sem);                                     \
         if (locked == 0)                                                            \
             uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_SHARED);                    \
+        else                                                                        \
+            uvm_assert_rwsem_locked_read(_sem);                                     \
         locked;                                                                     \
     })
 
+// Lock w/o any tracking. This should be extremely rare and *_no_tracking
+// helpers will be added only as needed.
+//
+// TODO: Bug 2594854:
+// TODO: Bug 2583279: Remove macro when bugs are fixed
+#define uvm_down_read_trylock_no_tracking(uvm_sem) ({                               \
+        typeof(uvm_sem) _sem = (uvm_sem);                                           \
+        down_read_trylock(&_sem->sem);                                              \
+    })
+
 // trylock for writing: returns 1 if successful, 0 if not.  Out-of-order lock
 // acquisition via this function is legal, i.e. the lock order checker will
 // allow it.  However, if an out-of-order lock acquisition attempt fails, it is
 // the caller's responsibility to back off at least to the point where the
 // next held lower-order lock is released.
-static int __uvm_down_write_trylock(uvm_rw_semaphore_t *uvm_sem)
-{
-    int ret = down_write_trylock(&uvm_sem->sem);
-    if (ret == 0)
-        return ret;
-
-    uvm_assert_rwsem_locked_write(uvm_sem);
-
-    return ret;
-}
-#define uvm_down_write_trylock(sem) ({                                                 \
-        typeof(sem) _sem = (sem);                                                      \
+#define uvm_down_write_trylock(uvm_sem) ({                                             \
+        typeof(uvm_sem) _sem = (uvm_sem);                                              \
         int locked;                                                                    \
         uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE | UVM_LOCK_FLAGS_TRYLOCK); \
-        locked = __uvm_down_write_trylock(_sem);                                       \
+        locked = down_write_trylock(&_sem->sem);                                       \
         if (locked == 0)                                                               \
             uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE);                    \
+        else                                                                           \
+            uvm_assert_rwsem_locked_write(_sem);                                       \
         locked;                                                                        \
     })
 
-static void __uvm_up_write(uvm_rw_semaphore_t *uvm_sem)
-{
-    uvm_assert_rwsem_locked_write(uvm_sem);
-    up_write(&uvm_sem->sem);
-}
-#define uvm_up_write(sem) ({                                    \
-        typeof(sem) _sem = (sem);                               \
-        __uvm_up_write(_sem);                                   \
+#define uvm_up_write(uvm_sem) ({                                \
+        typeof(uvm_sem) _sem = (uvm_sem);                       \
+        uvm_assert_rwsem_locked_write(_sem);                    \
+        up_write(&_sem->sem);                                   \
         uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
-static void __uvm_downgrade_write(uvm_rw_semaphore_t *uvm_sem)
-{
-    uvm_assert_rwsem_locked_write(uvm_sem);
-    downgrade_write(&uvm_sem->sem);
-}
-#define uvm_downgrade_write(sem) ({                     \
-        typeof(sem) _sem = (sem);                       \
-        __uvm_downgrade_write(_sem);                    \
+#define uvm_downgrade_write(uvm_sem) ({                 \
+        typeof(uvm_sem) _sem = (uvm_sem);               \
+        uvm_assert_rwsem_locked_write(_sem);            \
+        downgrade_write(&_sem->sem);                    \
         uvm_record_downgrade(_sem);                     \
     })
 
@@ -774,9 +765,17 @@ typedef struct
 #endif
 } uvm_mutex_t;
 
-#define uvm_assert_mutex_locked(uvm_mutex) ({                                                               \
-        typeof(uvm_mutex) _mutex = (uvm_mutex);                                                             \
-        UVM_ASSERT(mutex_is_locked(&_mutex->m) && uvm_check_locked(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)); \
+//
+// Note that this is a macro, not an inline or static function so the
+// "uvm_macro" argument is subsituted as text. If this is invoked with
+// uvm_assert_mutex_locked(_mutex) then we get code "_mutex = _mutex"
+// and _mutex is initialized to NULL. Avoid this by using a name unlikely to
+// be the same as the string passed to "uvm_mutex".
+// See uvm_mutex_lock() and uvm_mutex_unlock() below as examples.
+//
+#define uvm_assert_mutex_locked(uvm_mutex) ({                                                                 \
+        typeof(uvm_mutex) _mutex_ = (uvm_mutex);                                                              \
+        UVM_ASSERT(mutex_is_locked(&_mutex_->m) && uvm_check_locked(_mutex_, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)); \
     })
 
 #define uvm_assert_mutex_unlocked(uvm_mutex) UVM_ASSERT(!mutex_is_locked(&(uvm_mutex)->m))
@@ -791,34 +790,27 @@ static void uvm_mutex_init(uvm_mutex_t *mutex, uvm_lock_order_t lock_order)
     uvm_assert_mutex_unlocked(mutex);
 }
 
-static void __uvm_mutex_lock(uvm_mutex_t *mutex)
-{
-    mutex_lock(&mutex->m);
-    uvm_assert_mutex_locked(mutex);
-}
 #define uvm_mutex_lock(mutex) ({                                \
         typeof(mutex) _mutex = (mutex);                         \
         uvm_record_lock(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
-        __uvm_mutex_lock(_mutex);                               \
+        mutex_lock(&_mutex->m);                                 \
+        uvm_assert_mutex_locked(_mutex);                        \
     })
 
 // Lock w/o any tracking. This should be extremely rare and *_no_tracking
 // helpers will be added only as needed.
 #define uvm_mutex_lock_no_tracking(mutex) mutex_lock(&(mutex)->m)
 
-static void __uvm_mutex_unlock(uvm_mutex_t *mutex)
-{
-    uvm_assert_mutex_locked(mutex);
-    mutex_unlock(&mutex->m);
-}
 #define uvm_mutex_unlock(mutex) ({                                \
         typeof(mutex) _mutex = (mutex);                           \
-        __uvm_mutex_unlock(_mutex);                               \
+        uvm_assert_mutex_locked(_mutex);                          \
+        mutex_unlock(&_mutex->m);                                 \
         uvm_record_unlock(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 #define uvm_mutex_unlock_out_of_order(mutex) ({                                \
         typeof(mutex) _mutex = (mutex);                                        \
-        __uvm_mutex_unlock(_mutex);                                            \
+        uvm_assert_mutex_locked(_mutex);                                       \
+        mutex_unlock(&_mutex->m);                                              \
         uvm_record_unlock_out_of_order(_mutex, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
@@ -843,28 +835,20 @@ static void uvm_sema_init(uvm_semaphore_t *semaphore, int val, uvm_lock_order_t
 #endif
 }
 
-static void __uvm_down(uvm_semaphore_t *semaphore)
-{
-    down(&semaphore->sem);
-}
-#define uvm_down(sem) ({                                   \
-        typeof(sem) _sem = (sem);                          \
+#define uvm_down(uvm_sem) ({                               \
+        typeof(uvm_sem) _sem = (uvm_sem);                  \
         uvm_record_lock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
-        __uvm_down(_sem);                                  \
+        down(&_sem->sem);                                  \
     })
 
-static void __uvm_up(uvm_semaphore_t *semaphore)
-{
-    up(&semaphore->sem);
-}
-#define uvm_up(sem) ({                                       \
-        typeof(sem) _sem = (sem);                            \
-        __uvm_up(_sem);                                      \
+#define uvm_up(uvm_sem) ({                                   \
+        typeof(uvm_sem) _sem = (uvm_sem);                    \
+        up(&_sem->sem);                                      \
         uvm_record_unlock(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
     })
-#define uvm_up_out_of_order(sem) ({                                       \
-        typeof(sem) _sem = (sem);                                         \
-        __uvm_up(_sem);                                                   \
+#define uvm_up_out_of_order(uvm_sem) ({                                   \
+        typeof(uvm_sem) _sem = (uvm_sem);                                 \
+        up(&_sem->sem);                                                   \
         uvm_record_unlock_out_of_order(_sem, UVM_LOCK_FLAGS_MODE_SHARED); \
     })
 
@@ -893,9 +877,16 @@ typedef struct
 
 // Asserts that the spinlock is held. Notably the macros below support both
 // types of spinlocks.
-#define uvm_assert_spinlock_locked(spinlock) ({                                                             \
-        typeof(spinlock) _lock = (spinlock);                                                                \
-        UVM_ASSERT(spin_is_locked(&_lock->lock) && uvm_check_locked(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)); \
+// Note that this is a macro, not an inline or static function so the
+// "spinlock" argument is subsituted as text. If this is invoked with
+// uvm_assert_spinlock_locked(_lock) then we get code "_lock = _lock"
+// and _lock is initialized to NULL. Avoid this by using a name unlikely to
+// be the same as the string passed to "spinlock".
+// See uvm_spin_lock() and uvm_spin_unlock() below as examples.
+//
+#define uvm_assert_spinlock_locked(spinlock) ({                                                               \
+        typeof(spinlock) _lock_ = (spinlock);                                                                 \
+        UVM_ASSERT(spin_is_locked(&_lock_->lock) && uvm_check_locked(_lock_, UVM_LOCK_FLAGS_MODE_EXCLUSIVE)); \
     })
 
 #define uvm_assert_spinlock_unlocked(spinlock) UVM_ASSERT(!spin_is_locked(&(spinlock)->lock))
@@ -910,25 +901,17 @@ static void uvm_spin_lock_init(uvm_spinlock_t *spinlock, uvm_lock_order_t lock_o
     uvm_assert_spinlock_unlocked(spinlock);
 }
 
-static void __uvm_spin_lock(uvm_spinlock_t *spinlock)
-{
-    spin_lock(&spinlock->lock);
-    uvm_assert_spinlock_locked(spinlock);
-}
-#define uvm_spin_lock(lock) ({                                 \
-        typeof(lock) _lock = (lock);                           \
+#define uvm_spin_lock(uvm_lock) ({                             \
+        typeof(uvm_lock) _lock = (uvm_lock);                   \
         uvm_record_lock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
-        __uvm_spin_lock(_lock);                                \
+        spin_lock(&_lock->lock);                               \
+        uvm_assert_spinlock_locked(_lock);                     \
     })
 
-static void __uvm_spin_unlock(uvm_spinlock_t *spinlock)
-{
-    uvm_assert_spinlock_locked(spinlock);
-    spin_unlock(&spinlock->lock);
-}
-#define uvm_spin_unlock(lock) ({                                 \
-        typeof(lock) _lock = (lock);                             \
-        __uvm_spin_unlock(_lock);                                \
+#define uvm_spin_unlock(uvm_lock) ({                             \
+        typeof(uvm_lock) _lock = (uvm_lock);                     \
+        uvm_assert_spinlock_locked(_lock);                       \
+        spin_unlock(&_lock->lock);                               \
         uvm_record_unlock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
@@ -942,30 +925,22 @@ static void uvm_spin_lock_irqsave_init(uvm_spinlock_irqsave_t *spinlock, uvm_loc
     uvm_assert_spinlock_unlocked(spinlock);
 }
 
-static void __uvm_spin_lock_irqsave(uvm_spinlock_irqsave_t *spinlock)
-{
-    // Use a temp to not rely on flags being written after acquiring the lock.
-    unsigned long irq_flags;
-    spin_lock_irqsave(&(spinlock)->lock, irq_flags);
-    spinlock->irq_flags = irq_flags;
-    uvm_assert_spinlock_locked(spinlock);
-}
-#define uvm_spin_lock_irqsave(lock) ({                         \
-        typeof(lock) _lock = (lock);                           \
+// Use a temp to not rely on flags being written after acquiring the lock.
+#define uvm_spin_lock_irqsave(uvm_lock) ({                     \
+        typeof(uvm_lock) _lock = (uvm_lock);                   \
+        unsigned long irq_flags;                               \
         uvm_record_lock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
-        __uvm_spin_lock_irqsave(_lock);                        \
+        spin_lock_irqsave(&_lock->lock, irq_flags);            \
+        _lock->irq_flags = irq_flags;                          \
+        uvm_assert_spinlock_locked(_lock);                     \
     })
 
-static void __uvm_spin_unlock_irqrestore(uvm_spinlock_irqsave_t *spinlock)
-{
-    // Use a temp to not rely on flags being read before releasing the lock.
-    unsigned long irq_flags = spinlock->irq_flags;
-    uvm_assert_spinlock_locked(spinlock);
-    spin_unlock_irqrestore(&(spinlock)->lock, irq_flags);
-}
-#define uvm_spin_unlock_irqrestore(lock) ({                      \
-        typeof(lock) _lock = (lock);                             \
-        __uvm_spin_unlock_irqrestore(_lock);                     \
+// Use a temp to not rely on flags being read before releasing the lock.
+#define uvm_spin_unlock_irqrestore(uvm_lock) ({                  \
+        typeof(uvm_lock) _lock = (uvm_lock);                     \
+        unsigned long irq_flags = _lock->irq_flags;              \
+        uvm_assert_spinlock_locked(_lock);                       \
+        spin_unlock_irqrestore(&_lock->lock, irq_flags);         \
         uvm_record_unlock(_lock, UVM_LOCK_FLAGS_MODE_EXCLUSIVE); \
     })
 
