@@ -612,7 +612,7 @@ static NvBool nv_dma_is_map_resource_implemented
 #endif
     }
 
-    return (ops && ops->map_resource);
+    return (ops->map_resource != NULL);
 #else
     return NV_FALSE;
 #endif
