@@ -16,6 +16,12 @@
 
 #include "nv-time.h"
 
+
+
+
+
+
+
 extern char *NVreg_TemporaryFilePath;
 
 #define MAX_ERROR_STRING 512
@@ -1111,24 +1117,6 @@ NvU32 NV_API_CALL os_get_cpu_count()
     return NV_NUM_CPUS();
 }
 
-void NV_API_CALL os_register_compatible_ioctl(NvU32 cmd, NvU32 size)
-{
-#if defined(NVCPU_X86_64) && defined(CONFIG_IA32_EMULATION) && \
-  !defined(NV_FILE_OPERATIONS_HAS_COMPAT_IOCTL)
-    unsigned int request = _IOWR(NV_IOCTL_MAGIC, cmd, char[size]);
-    register_ioctl32_conversion(request, (void *)sys_ioctl);
-#endif
-}
-
-void NV_API_CALL os_unregister_compatible_ioctl(NvU32 cmd, NvU32 size)
-{
-#if defined(NVCPU_X86_64) && defined(CONFIG_IA32_EMULATION) && \
-  !defined(NV_FILE_OPERATIONS_HAS_COMPAT_IOCTL)
-    unsigned int request = _IOWR(NV_IOCTL_MAGIC, cmd, char[size]);
-    unregister_ioctl32_conversion(request);
-#endif
-}
-
 BOOL NV_API_CALL os_pat_supported(void)
 {
     return (nv_pat_mode != NV_PAT_MODE_DISABLED);
@@ -2179,3 +2167,36 @@ retry:
     return NV_ERR_NOT_SUPPORTED;
 #endif
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
